# pysolr imports
from pysolr import Solr

# lfs imports
from lfs.catalog.models import Product
from lfs.catalog.settings import STANDARD_PRODUCT
from lfs.catalog.settings import PRODUCT_WITH_VARIANTS
from lfs.catalog.settings import CONFIGURABLE_PRODUCT

try:
    SOLR_ADDRESS = settings.SOLR_ADDRESS
except:
    from lfs_solr.settings import SOLR_ADDRESS

def index_product(product):
    """Indexes passed product.
    """
    if product.is_variant():
        try:
            product = product.parent.get_default_variant()
        except AttributeError:
            return

    _index_products([product])

def delete_product(product):
    """Deletes passed product from index.
    """
    conn = Solr(SOLR_ADDRESS)
    conn.delete(id=product.id)
    
def index_all_products():
    """Indexes all products.
    """
    products = Product.objects.filter(
        active=True, sub_type__in = (STANDARD_PRODUCT, PRODUCT_WITH_VARIANTS, CONFIGURABLE_PRODUCT))

    _index_products(products, delete=True)

def _index_products(products, delete=False):
    """Indexes given products.
    """
    conn = Solr(SOLR_ADDRESS)
    if delete:
        conn.delete(q='*:*')

    temp = []
    for product in products:

        # Just index the default variant of a "Product with Variants"
        if product.is_product_with_variants():
            product = product.get_default_variant()

        if product is None:
            continue

        # Categories
        categories = []
        for category in product.get_categories():
            categories.append(category.name)

        # Manufacturer
        manufacturer = product.manufacturer
        if manufacturer:
            manufacturer_name = manufacturer.name
        else:
            manufacturer_name = ""

        temp.append({
            "id" : product.id,
            "name" : product.get_name(),
            "price" : product.get_price(),
            "categories" : categories,
            "keywords" : product.get_meta_keywords(),
            "manufacturer" : manufacturer_name,
            "sku_manufacturer" : product.sku_manufacturer,
            "description" : product.description,
        })

    conn.add(temp)


# django imports
from django.db.models.signals import post_save
from django.db.models.signals import post_delete

# lfs imports
from lfs.catalog.models import Product
from lfs.core.signals import product_changed

# lfs_solr imports
from lfs_solr.utils import index_product
from lfs_solr.utils import delete_product

def product_deleted_listener(sender, instance, **kwargs):
    delete_product(instance)
post_delete.connect(product_deleted_listener, sender=Product)

def product_saved_listener(sender, instance, **kwargs):
    if instance.is_active():
        index_product(instance)
    else:
        delete_product(instance)
post_save.connect(product_saved_listener, sender=Product)




from setuptools import setup, find_packages
import os

version = '1.0b1'

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.txt')).read()

setup(name='lfs-solr',
      version=version,
      description='Solr integration for LFC',
      long_description=README,
      classifiers=[
          'Development Status :: 4 - Beta',
          'Environment :: Web Environment',
          'Framework :: Django',
          'License :: OSI Approved :: BSD License',
          'Operating System :: OS Independent',
          'Programming Language :: Python',
      ],
      keywords='django lfs search solr',
      author='Kai Diefenbach',
      author_email='kai.diefenbach@iqpp.de',
      url='http://www.iqpp.de',
      license='BSD',
      packages=find_packages(exclude=['ez_setup']),
      include_package_data=True,
      zip_safe=False,
      install_requires=[
        'setuptools',
        'pysolr',
      ],
      )



# django imports
from django.conf.urls.defaults import *

urlpatterns = patterns('lfs_solr.views',
    url(r'^search', "search", name="solr_search"),
    url(r'^livesearch', "livesearch", name="solr_livesearch"),
    url(r'^solr-set-filter', "set_filter", name="solr_set_filter"),
    url(r'^solr-set-sorting', "set_sorting", name="solr_set_sorting"),
    url(r'^solr-reset-filter', "reset_filter", name="solr_reset_filter"),
    url(r'^solr-reset-field', "reset_field", name="solr_reset_field"),
    url(r'^index-products', "index_products", name="solr_index_products"),
)


# coding: utf-8

# django imports
from django.conf import settings
from django.contrib.auth.decorators import permission_required
from django.core.urlresolvers import reverse
from django.http import HttpResponse
from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response
from django.template.loader import render_to_string
from django.template import RequestContext
from django.utils import simplejson

# pysolr imports
from pysolr import Solr

# lfs imports
from lfs.catalog.models import Product

# lfs_solr

try:
    SOLR_ADDRESS = settings.SOLR_ADDRESS
except:
    from lfs_solr.settings import SOLR_ADDRESS

@permission_required("manage_shop", login_url="/login/")
def index_products(request):
    """Indexes full all products.
    """
    from lfs_solr.utils import index_all_products
    index_all_products()

    return HttpResponse("Done!")

def reset_field(request):
    """Resets filter for given field.
    """
    q = request.GET.get("q")
    field = request.GET.get("field")
    try:
        solr_filter = request.session["solr_filter"]
        del solr_filter[field]
    except KeyError:
        pass
    else:
        request.session["solr_filter"] = solr_filter

    return HttpResponseRedirect(reverse("solr_search") + "?q=" + q)

def reset_filter(request):
    """Resets all filters.
    """
    q = request.GET.get("q")
    try:
       del request.session["solr_filter"]
    except KeyError:
        pass

    return HttpResponseRedirect(reverse("solr_search") + "?q=" + q)

def set_filter(request):
    """Saves the filter for given field to current session.
    """
    field = request.GET.get("field")
    value = request.GET.get("value")
    q = request.GET.get("q")

    if request.session.has_key("solr_filter") == False:
        request.session["solr_filter"] = {}

    solr_filter = request.session["solr_filter"]
    solr_filter[field] = value

    request.session["solr_filter"]  = solr_filter
    return HttpResponseRedirect(reverse("solr_search") + "?q=" + q)

def set_sorting(request):
    """Saves the sorting to current session.
    """
    q = request.GET.get("q")
    sorting = request.GET.get("sorting")

    if sorting:
        request.session["solr_sorting"] = sorting
    else:
        try:
            del request.session["solr_sorting"]
        except KeyError:
            pass

    return HttpResponseRedirect(reverse("solr_search") + "?q=" + q)

def livesearch(request, template_name="lfs_solr/livesearch_results.html"):
    """Renders the results for the live search.
    """
    rows = 10
    q = request.GET.get("q", "")

    if q == "":
        result = simplejson.dumps({
            "state" : "failure",
        })
    else:
        conn = Solr(SOLR_ADDRESS)

        params = {
          'rows' : rows,
        }

        results = conn.search(q.lower(), **params)

        # Products
        products = []
        for doc in results.docs:
            product = Product.objects.get(pk=doc["id"])
            products.append(product)

        products = render_to_string(template_name, RequestContext(request, {
            "products" : products,
            "q" : q,
            "total" : results.hits,
        }))

        result = simplejson.dumps({
            "state" : "success",
            "products" : products,
        })

    return HttpResponse(result)

def search(request, template_name="lfs_solr/search_results.html"):
    """Provides form and result for search via Solr.
    """
    if request.GET.get("reset") or request.GET.get("livesearch"):
        try:
           del request.session["solr_filter"]
        except KeyError:
            pass

    rows = 10
    q = request.GET.get("q")

    try:
        start = int(request.GET.get("start", 0))
    except ValueError:
        start = 0

    if q:
        conn = Solr(SOLR_ADDRESS)

        params = {
          'facet': 'on',
          'facet.field': ['categories', 'manufacturer'],
          'facet.mincount' : 1,
          'rows' : rows,
          "start" : start,
        }

        # Sorting
        sorting = request.session.get("solr_sorting")
        if sorting:
            params["sort"] = sorting

        if request.session.has_key("solr_filter"):
            params["fq"] = []
            for field, value in request.session["solr_filter"].items():
                params["fq"].append("%s:%s" % (field.encode("utf-8"), value.encode("utf-8")))

        results = conn.search(q.lower(), **params)

        # Products
        products = []
        for doc in results.docs:
            try:
                product = Product.objects.get(pk=doc["id"])
            except Product.DoesNotExist:
                pass
            else:
                products.append(product)

        # Facets
        categories = []
        temp = results.facets["facet_fields"]["categories"]
        for i in range(1, len(temp), 2):
            name = temp[i-1]
            if name.find(" "):
                url = '"%s"' % name
            else:
                url = name
            categories.append({
                "url" : url,
                "name" : name,
                "amount" : temp[i],
            })

        manufacturers = []
        temp = results.facets["facet_fields"]["manufacturer"]
        for i in range(1, len(temp), 2):
            name = temp[i-1]
            if name:
                if name.find(" "):
                    url = '"%s"' % name
                else:
                    url = name
                manufacturers.append({
                    "url" : url,
                    "name" : name,
                    "amount" : temp[i],
                })

        # Pagination
        if start + rows < results.hits:
            display_next = True
            next_start = start + rows
        else:
            display_next = False
            next_start = None

        if start-rows >= 0:
            display_previous = True
            previous_start = start - rows
        else:
            display_previous = False
            previous_start = None

        return render_to_response(template_name, RequestContext(request, {
            "products" : products,
            "results" : results,
            "categories" : categories,
            "manufacturers" : manufacturers,
            "categories_reset" : "categories" in request.session.get("solr_filter", []),
            "manufacturer_reset" : "manufacturer" in request.session.get("solr_filter", []),
            "total" : results.hits,
            "display_next" : display_next,
            "next_start" : next_start,
            "previous_start" : previous_start,
            "display_next" : display_next,
            "display_previous" : display_previous,
            "q" : q,
            "sorting" : sorting,
        }))
    else:
        return render_to_response(template_name, RequestContext(request, {}))